@@ -3,6 +3,7 @@
 from pagador.configuracao.cadastro import ValoresBase
 from pagador.extensao.base import Plugavel, PluginNaoEncontrado
 from repositories.configuracao import models
+from repositories.configuracao.models import PagamentoBanco
 
 
 class FormaDePagamento(models.FormaPagamento, Plugavel):
@@ -12,6 +13,7 @@ class FormaDePagamento(models.FormaPagamento, Plugavel):
                 pg.pagamento_codigo,
                 pg.pagamento_ativado,
                 pg.pagamento_plano_indice,
+                pg.pagamento_parcela_valor_minimo_parcela,
                 pgc.pagamento_configuracao_id,
                 pgc.pagamento_configuracao_usuario,
                 pgc.pagamento_configuracao_senha,
@@ -50,16 +52,21 @@ class FormaDePagamento(models.FormaPagamento, Plugavel):
 
     @property
     def bancos_ativos_na_conta(self):
-        bancos_ativos = self.bancos.filter(conta=self.conta, ativo=True)
-        if bancos_ativos:
-            return [{"id": banco.id, "nome": banco.banco.nome, "imagem": banco.banco.imagem, "codigo": banco.banco.codigo} for banco in bancos_ativos]
-        return None
+        return [{"id": banco.id, "nome": banco.banco.nome, "imagem": banco.banco.imagem, "codigo": banco.banco.codigo} for banco in self.pagamento_bancos if banco.ativo]
+
+    @property
+    def pagamento_bancos(self):
+        if not hasattr(self, "_pagamento_bancos"):
+            self._pagamento_bancos = None
+        if self._pagamento_bancos is None:
+            self._pagamento_bancos = [banco for banco in PagamentoBanco.objects.prefetch_related("banco").filter(pagamento_id=self.id, conta_id=self.conta_id)]
+        return self._pagamento_bancos
 
     @property
     def configurado(self):
         try:
             try:
-                bancos = self.bancos.all()[0]
+                bancos = self.pagamento_bancos[0]
             except IndexError:
                 bancos = None
             return (self.pagamento_configuracao_usuario or
@@ -80,7 +87,7 @@ class FormaDePagamento(models.FormaPagamento, Plugavel):
             cls.QUERY_CONFIGURACOES_ATIVAS.format((" AND pg.pagamento_id=%s" if pagamento_id else '')), parametros
         )
 
-    def to_dict(self, com_configuracao=True, plano_indice=None):
+    def to_dict(self, com_configuracao=True, plano_indice=None, soh_habilitados=False):
         forma_pagamento = {
             "id": self.id,
             "codigo": self.codigo,
@@ -102,31 +109,57 @@ class FormaDePagamento(models.FormaPagamento, Plugavel):
             if plano_indice:
                 forma_pagamento["habilitado_na_loja"] = forma_pagamento["configurado"] and self.ativo and self.pagamento_configuracao_ativo and self.plano_indice <= plano_indice
             forma_pagamento["bancos_ativos_na_conta"] = self.bancos_ativos_na_conta
-
+        if soh_habilitados and "habilitado_na_loja" in forma_pagamento and not forma_pagamento["habilitado_na_loja"]:
+            return None
         return forma_pagamento
 
 
+class PagamentoNaoConfigurado(Exception):
+    pass
+
+
 class FormaDePagamentoConfiguracao(models.FormaPagamentoConfiguracao, Plugavel):
     class Meta:
         proxy = True
 
     @property
     def meio_pagamento(self):
-        return self.forma_pagamento.codigo
+        if "codigo" in self.forma_de_pagamento:
+            return self.forma_de_pagamento["codigo"]
+        raise PagamentoNaoConfigurado("A forma de pagamento_id {} para a conta_id {} não foi configurada corretamente. Verifique com o suporte.".format(self.forma_pagamento_id, self.conta_id))
+
+    @property
+    def forma_de_pagamento(self):
+        if not hasattr(self, "_forma_de_pagamento"):
+            self._forma_de_pagamento = None
+        if self._forma_de_pagamento is None:
+            try:
+                self._forma_de_pagamento = FormaDePagamento.configuracoes_ativas(self.conta_id, pagamento_id=self.forma_pagamento_id)[0].to_dict()
+            except IndexError:
+                self._forma_de_pagamento = {}
+        return self._forma_de_pagamento
 
     def bancos_configurados_na_conta(self, banco_id=None):
-        if banco_id:
-            return models.PagamentoBanco.objects.get_or_create(conta_id=self.conta_id, pagamento=self.forma_pagamento, banco_id=banco_id)
-        return models.PagamentoBanco.objects.filter(conta_id=self.conta_id, pagamento=self.forma_pagamento)
+        if not hasattr(self, "_bancos_configurados"):
+            self._bancos_configurados = None
+        if self._bancos_configurados is None:
+            if banco_id:
+                self._bancos_configurados = models.PagamentoBanco.objects.get_or_create(conta_id=self.conta_id, pagamento_id=self.forma_pagamento_id, banco_id=banco_id)
+            else:
+                self._bancos_configurados = models.PagamentoBanco.objects.filter(conta_id=self.conta_id, pagamento_id=self.forma_pagamento_id)
+        return self._bancos_configurados
 
     @property
     def bancos(self):
-        bancos = models.Banco.objects.all()
-        for banco in bancos:
-            for pagamento_banco in self.bancos_configurados_na_conta():
-                if banco.id == pagamento_banco.banco_id:
-                    banco.pagamento_banco = pagamento_banco
-        return bancos
+        if not hasattr(self, "_bancos"):
+            self._bancos = None
+        if self._bancos is None:
+            self._bancos = models.Banco.objects.all()
+            for banco in self._bancos:
+                for pagamento_banco in self.bancos_configurados_na_conta():
+                    if banco.id == pagamento_banco.banco_id:
+                        banco.pagamento_banco = pagamento_banco
+            return self._bancos
 
     def save(self, *args, **kwargs):
         self.desconto_tipo = self.TIPO_PORCENTAGEM
@@ -152,7 +185,7 @@ class FormaDePagamentoConfiguracao(models.FormaPagamentoConfiguracao, Plugavel):
             try:
                 self._formulario = self.plugin_configuracao.Formulario()
             except AttributeError:
-                raise PluginNaoEncontrado(u"O meio de pagamento {} (id={}) não possui a classe configuracao.Formulario.".format(self.forma_pagamento.codigo, self.forma_pagamento.id))
+                raise PluginNaoEncontrado(u"O meio de pagamento {} (id={}) não possui a classe configuracao.Formulario.".format(self.meio_pagamento, self.forma_pagamento_id))
         return self._formulario
 
     @property
@@ -160,14 +193,14 @@ class FormaDePagamentoConfiguracao(models.FormaPagamentoConfiguracao, Plugavel):
         try:
             return self.plugin_configuracao.MeioPagamentoEnvio()
         except AttributeError:
-            raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe configuracao.MeioPagamentoEnvio.".format(self.forma_pagamento.codigo))
+            raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe configuracao.MeioPagamentoEnvio.".format(self.meio_pagamento))
 
     @property
     def selecao(self):
         try:
             return self.plugin_configuracao.MeioPagamentoSelecao(self)
         except AttributeError:
-            raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe configuracao.MeioPagamentoSelecao.".format(self.forma_pagamento.codigo))
+            raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe configuracao.MeioPagamentoSelecao.".format(self.meio_pagamento))
 
     def instalar(self, dados):
         dados_instalacao = self.instalador.dados_de_instalacao(dados)
@@ -201,14 +234,14 @@ class FormaDePagamentoConfiguracao(models.FormaPagamentoConfiguracao, Plugavel):
         try:
             return self.plugin_seguranca.Instalador(self, id=self.conta_id)
         except AttributeError, ex:
-            raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe seguranca.Instalador.".format(self.forma_pagamento.codigo))
+            raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe seguranca.Instalador.".format(self.meio_pagamento))
 
     @property
     def valores(self):
         try:
             return ValoresBase(self, self.formulario)
         except AttributeError:
-            raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe configuracao.MeioPagamentoValores.".format(self.forma_pagamento.codigo))
+            raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe configuracao.MeioPagamentoValores.".format(self.meio_pagamento))
 
     @property
     def cadastro(self):
@@ -217,7 +250,7 @@ class FormaDePagamentoConfiguracao(models.FormaPagamentoConfiguracao, Plugavel):
         except AttributeError:
             return None
             #FIXME: Pensar em um modelo melhor para determinar extensoes não obrigatórias
-            # raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe configuracao.MeioPagamentoCadastro.".format(self.forma_pagamento.codigo))
+            # raise PluginNaoEncontrado(u"O meio de pagamento '{}' não possui a classe configuracao.MeioPagamentoCadastro.".format(self.meio_pagamento))
 
     @property
     def eh_aplicacao(self):
@@ -230,21 +263,27 @@ class FormaDePagamentoConfiguracao(models.FormaPagamentoConfiguracao, Plugavel):
     def salvar(self, dados):
         self.formulario.define_valores_em_model(self, dados).save()
 
-    def salvar_complemento(self, dados):
-        return self.cadastro.salvar_complemento(dados)
+    def complemento(self, dados):
+        return self.cadastro.complemento(dados)
 
     def to_dict(self):
         cadastro = None
         if self.cadastro:
             cadastro = self.cadastro.to_dict()
-        try:
-            forma_pagamento = FormaDePagamento.configuracoes_ativas(self.conta.id, pagamento_id=self.forma_pagamento.id)[0].to_dict()
-        except IndexError:
-            forma_pagamento = {}
         return {
             "cadastro": cadastro,
             "valores": self.valores.to_dict(),
             "formulario": self.formulario.to_dict(),
-            "pagamento": forma_pagamento,
+            "pagamento": self.forma_de_pagamento,
             "eh_aplicacao": self.eh_aplicacao
         }
+
+
+class Banco(models.Banco):
+    class Meta:
+        proxy = True
+
+
+class BoletoCarteira(models.BoletoCarteira):
+    class Meta:
+        proxy = True
