@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 from urllib import quote_plus
 from xml.etree import ElementTree
+from decimal import Decimal
 
 
 class Formatador(object):
@@ -14,6 +15,13 @@ class Formatador(object):
     def string_para_url(cls, texto):
         return quote_plus(unicode(texto))
 
+    @classmethod
+    def string_para_decimal(cls, valor):
+        try:
+            return Decimal(valor)
+        except ValueError:
+            return Decimal(0)
+
     @classmethod
     def formata_decimal(cls, valor, como_float=False, como_int=False):
         if como_float:
